package src;
public class Philosopher implements Runnable {
    private final Chopstick leftChopstick;
    private final Chopstick rightChopstick;
    private final Arbitrator arbitrator;

    public Philosopher(Chopstick leftChopstick, Chopstick rightChopstick, Arbitrator arbitrator) {
        this.leftChopstick = leftChopstick;
        this.rightChopstick = rightChopstick;
        this.arbitrator = arbitrator;
    }

    @Override
    public void run() {
        try {
            while (true) {
                think();
                arbitrator.requestPermission(this);  // Request permission from the arbitrator
                synchronized (leftChopstick) {
                    leftChopstick.pickUp();
                    synchronized (rightChopstick) {
                        rightChopstick.pickUp();
                        eat();
                        rightChopstick.putDown();
                    }
                    leftChopstick.putDown();
                }
                arbitrator.releasePermission(this);  // Release permission
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }

    private void think() {
        System.out.println(Thread.currentThread().getName() + " is thinking.");
    }

    private void eat() {
        System.out.println(Thread.currentThread().getName() + " is eating.");
    }
}

