package pp;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.Test;

class CounterTest {
    private final static int RUNS = 500000;

    private static void testIncrement(ICounter counter) {
        assertEquals(0, counter.get());
        var t1 = new Thread(() -> {
            for (var j = 0; j < RUNS; j++) {
                counter.inc();
            }
        });
        var t2 = new Thread(() -> {
            for (var j = 0; j < RUNS; j++) {
                counter.inc();
            }
        });
        t1.start();
        t2.start();
        try {
            t1.join();
            t2.join();
        } catch (InterruptedException ex) {
            Thread.currentThread().interrupt();
        }
        assertEquals(RUNS * 2, counter.get());
    }

    @Test
    void testCounterIncrement() throws InterruptedException {
        testIncrement(new Counter(0));
    }

    @Test
    void testSynchronizedCounterIncrement() throws InterruptedException {
        testIncrement(new SynchronizedCounter(0));
    }

    @Test
    void testAtomicCounter1Increment() throws InterruptedException {
        testIncrement(new AtomicCounter1(0));
    }

    @Test
    void testAtomicCounter2Increment() throws InterruptedException {
        testIncrement(new AtomicCounter2(0));
    }

    @Test
    void testAtomicCounter3Increment() throws InterruptedException {
        testIncrement(new AtomicCounter3(0));
    }

    @Test
    void testAtomicCounter4Increment() throws InterruptedException {
        testIncrement(new AtomicCounter4(0));
    }

    private static void testDecrement(ICounter counter) {
        assertEquals(RUNS * 2, counter.get());
        var t1 = new Thread(() -> {
            for (var j = 0; j < RUNS; j++) {
                counter.dec();
            }
        });
        var t2 = new Thread(() -> {
            for (var j = 0; j < RUNS; j++) {
                counter.dec();
            }
        });
        t1.start();
        t2.start();
        try {
            t1.join();
            t2.join();
        } catch (InterruptedException ex) {
            Thread.currentThread().interrupt();
        }
        assertEquals(0, counter.get());
    }

    @Test
    void testCounterDecrement() throws InterruptedException {
        testDecrement(new Counter(RUNS * 2));
    }

    @Test
    void testSynchronizedCounterDecrement() throws InterruptedException {
        testDecrement(new SynchronizedCounter(RUNS * 2));
    }

    @Test
    void testAtomicCounter1Decrement() throws InterruptedException {
        testDecrement(new AtomicCounter1(RUNS * 2));
    }

    @Test
    void testAtomicCounter2Decrement() throws InterruptedException {
        testDecrement(new AtomicCounter2(RUNS * 2));
    }

    @Test
    void testAtomicCounter3Decrement() throws InterruptedException {
        testDecrement(new AtomicCounter3(RUNS * 2));
    }

    @Test
    void testAtomicCounter4Decrement() throws InterruptedException {
        testDecrement(new AtomicCounter4(RUNS * 2));
    }

}
